-------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------
-- Recording when the venue has started
-------------------------------------------------------------------------------------------------
local venueStarted = false
local autochefEnabled = isAutochefEnabled()

local currentTutorial = getCurrentTutorial()
local noTutorialActive = currentTutorial == ""
local venueItem = getVenueInstance()
local flo = venueItem.getFlo()

local getNoTutorialActive = function()
    local testTutorial = getCurrentTutorial()
    local noTutorialActive = testTutorial == ""
    return noTutorialActive
end

local noodleSlots = {1, 2}
local noodleStepName = "TA_Noodle"
local taiyakiStepName = "TA_Taiyaki"


registerMessageHandler("kVenueStarted", "", {},
    function()
        venueStarted = true
        for idx = 1, #noodleSlots do
            local noodleFoodItem = venueItem.createFoodByIngredientID("TA_Noodle")
            local noodleAnchor = getStationWithId(noodleStepName..idx.."Anchor2")
            noodleAnchor.captureFoodItem(noodleFoodItem)
        end
    end
)


-------------------------------------------------------------------------------------------------
-- Soft Serve Machines Scripts
-------------------------------------------------------------------------------------------------
local numTapsArray = {0,0}
local reqNumTapsArray = {6,6}
local stepsArray = {"TA_Matcha","TA_BlackSesame"}
local animNumArray = {0,0}
local needsRefillArray = {2,2}

-- cycle between the station's first 6 animations until the second to last tap, don't do anything if the last tap triggers the refill
local setMachineAnimation = function(step, taps, tapsRequired, idx)
    animNumArray[idx] = animNumArray[idx] + 1
    if animNumArray[idx] > 6 then
        animNumArray[idx] = 1
    end
    local soundIdx = animNumArray[idx] % 2
    sendStationMsgWithParamsAndUserData("kActivate", { step.."Dispenser",animNumArray[idx],soundIdx}, {})
end

-- Set up a message handler for the refill tap areas for each soft serve station
--      Cycle the station's animations and accumulate taps until the required number,
--      then refill the station and reset the count.
local tapsRequired = 3

for idx = 1, #stepsArray do
    local step = stepsArray[idx]
    local dispenser = getStationWithId(step.."Dispenser")
    local tapArea = getStationWithId(step.."DispenserTapArea")
    local counterNode = getStationWithId(step.."Counter")

    if not(isNullRef(dispenser)) then

        local upgradeTaps = dispenser.getStationVal("NUM_TAPS")
        reqNumTapsArray[idx] = upgradeTaps - 1
        counterNode.setCurrentValue(1)
        counterNode.setCurrentValue(0)

        registerMessageHandler("kActivate", step.."DispenserRefillTapArea", { "tap" },
            function()
                if needsRefillArray[idx] >= 2 then
                    numTapsArray[idx] = numTapsArray[idx] + 1
                    setMachineAnimation(step, numTapsArray[idx], reqNumTapsArray[idx], idx)
                    if numTapsArray[idx] > reqNumTapsArray[idx] then
                        sendStationMsgWithParamsAndUserData("kActivate", { step.."DispenserRefill" }, {})
                        numTapsArray[idx] = 0
                        needsRefillArray[idx] = 0
                    end
                end
            end
        )

        registerMessageHandler("kStationCounterAtMinValue", step.."Counter", {},
            function()
                needsRefillArray[idx] = needsRefillArray[idx] + 1
            end
        )

        registerMessageHandler("kEnded", step.."DispenserTimer", {},
            function()
                needsRefillArray[idx] = needsRefillArray[idx] + 1
                sendStationMsgWithParamsAndUserData("kActivate", { "LuaTest allow refill",step, needsRefillArray[idx]}, {})
            end
        )
    end
end
-------------------------------------------------------------------------------------------------
-- End of Soft Serve Machines Scripts
-------------------------------------------------------------------------------------------------


-------------------------------------------------------------------------------------------------
-- Crane Machine Scripts
-------------------------------------------------------------------------------------------------
local stepName = "TA_Taiyaki"
local caught = false
local present = false
local started = false
local endedWithTap = false
local taiyakiDispenser = getStationWithId(stepName.."1Dispenser")
local craneTapArea = getStationWithId(stepName.."1TapArea")
local taiyakiUpgradeable = getStationWithId("TA_TaiyakiIngredientUpgradeable")
local taiyakiIngredientLevel = taiyakiUpgradeable.getUpgradeSpecificParam("ingredient_display_level")

-- Dopefish Easter Egg
local dopefishSprite = getStationWithId("DopefishSprite")
local dopefishVisible = false
local defaultTaiyakiSpritePath = "venue_32/flashatlas_v32/ingredients.cdx_taiyaki_ingredient."
local craneUseCount = 0
local craneUseDope = 6

if taiyakiIngredientLevel == "1" then
    defaultTaiyakiSpritePath = defaultTaiyakiSpritePath.."basic"
else
    defaultTaiyakiSpritePath = defaultTaiyakiSpritePath.."upgrade"..taiyakiIngredientLevel
end

local showDopefish = function(shouldShow)
    if not(isNullRef(dopefishSprite)) then
        local spritePath = defaultTaiyakiSpritePath
        if shouldShow then
            spritePath = "venue_32/atlas/recipe_icons/dopefish"
        end
        if not(shouldShow == dopefishVisible) then
            dopefishSprite.setSpritePath(spritePath)
            dopefishVisible = shouldShow
        end
    end
end
-- End of Dopefish Easter Egg

sendStationMsgWithParamsAndUserData("kActivate", { stepName.."IngredientUpgrade", taiyakiIngredientLevel}, {})

local resetStation = function()
    started = false
    caught = false
    present = false
    endedWithTap = false
    craneTapArea.setNoQueue(true)
    craneTapArea.setNoNavigate(true)
end

registerMessageHandler("kActivate", stepName.."1TapArea", { "visit" },
    function()
        if not(caught) and not(present) and not(started) then
            started = true
            sendStationMsgWithParamsAndUserData("kActivate", { stepName.."CraneStart" }, {})
        end
    end
)

registerMessageHandler("kActivate", stepName.."1TapArea", { "tap" },
    function()
        if present then
            --craneTapArea.moveFloTo()
            --flo.navigateToTapArea(craneTapArea)
        elseif caught and not(endedWithTap) then
            endedWithTap = true
            sendStationMsgWithParamsAndUserData("kActivate", { stepName.."CraneCatch" }, {})
            craneTapArea.setNoQueue(false)
            craneTapArea.setNoNavigate(false)
            craneUseCount = craneUseCount + 1
            showDopefish(craneUseCount == craneUseDope)
            if craneUseCount > craneUseDope then
                craneUseCount = 0
            end
        elseif not(started) then
            started = true
            sendStationMsgWithParamsAndUserData("kActivate", { stepName.."CraneStart" }, {})
        else
            sendStationMsgWithParamsAndUserData("kActivate", { stepName.."CraneTap" }, {})
        end
    end
)

registerMessageHandler("kStarted", stepName.."1WaitingTimer", {},
    function()
        started = true
        sendStationMsgWithParamsAndUserData("kActivate", { stepName.."CraneActive" }, {})
    end
)

registerMessageHandler("kEnded", stepName.."1WaitingTimer", {},
    function()
        started = true
        caught = true
        sendStationMsgWithParamsAndUserData("kActivate", { stepName.."CranePickup" }, {})
    end
)

registerMessageHandler("kEnded", stepName.."1CatchTimer", {},
    function()
        if not(endedWithTap) then
            resetStation()
            sendStationMsgWithParamsAndUserData("kActivate", { stepName.."CraneDrop" }, {})
        end
    end
)

registerMessageHandler("kIngredientReady", stepName.."1Dispenser", {},
    function()
        present = true
    end
)

registerMessageHandler("kIngredientOut", stepName.."1Dispenser", {},
    function()
        resetStation()
    end
)

-------------------------------------------------------------------------------------------------
-- End of Crane Machine Scripts
-------------------------------------------------------------------------------------------------
