-------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------
-- Recording when the venue has started
-------------------------------------------------------------------------------------------------
local venueStarted = false
local tutorialActive = false
local autochefEnabled = isAutochefEnabled()

local kNormalShakers = 2
-------------------------------------------------------------------------------------------------
-- Chopping Station Start
-------------------------------------------------------------------------------------------------
local choppingIngredientIds   = {"TK_Coconut","TK_Strawberry","TK_Pineapple","TK_Orange"}
local choppingInstantMods     = {"TK_CoconutChips","TK_SlicedStrawberry","TK_PineappleSpear","TK_OrangeWheel"}
local ingredientIdToStepMap   = {}
local storageSlotsQueue = {}
local maxStorageSlots = 0


local addSlotToQueue = function(slot)
    table.insert(storageSlotsQueue,1,slot)
end

local removeSlotFromQueue = function(slot)
    for idx = 1,#storageSlotsQueue do
        local queueSlot = storageSlotsQueue[idx]
        if queueSlot == slot then
            table.remove(storageSlotsQueue, idx)
            return
        end
    end
end

local bumpOldestInQueue = function()
    --Remove any elements over the max queue size
    local oldestSlot = storageSlotsQueue[#storageSlotsQueue]
    local idx = #storageSlotsQueue

    while idx >= maxStorageSlots do
        oldestSlot = storageSlotsQueue[idx]
        table.remove(storageSlotsQueue,idx)

        idx = #storageSlotsQueue
        --now remove the item from this storage slot, this should kick off the transfer from the chopping station
        oldestSlot.removeItem()

    end
end

for idx = 1, #choppingIngredientIds do
    local ingredientId = choppingIngredientIds[idx]
    ingredientIdToStepMap[ingredientId] = choppingInstantMods[idx]
end

registerMessageHandler("kVenueStarted", "", {},
	function()
		venueStarted = true
		displayCookie = true
	end
)

for idx = 1,3 do
    local instantModSlotId = "TK_ChoppedStorageSlotInstantMod"..idx
    local storageSlotId = "TK_ChoppedStorageSlot"..idx
    local instantModSlot = getStationWithId(instantModSlotId)
    local storageSlot = getStationWithId(storageSlotId)
    if storageSlot.getUnlocked() then
        maxStorageSlots = maxStorageSlots + 1

        registerMessageHandler("kIngredientIn", storageSlotId, {},
            function()
                local storedIngredient = storageSlot.getFoodItem()
                if not isNullRef(storedIngredient) then
                    local ingredientId = storedIngredient.getFirstIngredientID()
                    instantModSlot.setStepName(ingredientIdToStepMap[ingredientId])
                    addSlotToQueue(storageSlot)
                end
            end
        )

        registerMessageHandler("kStepApplied", instantModSlotId, {},
            function()
                instantModSlot.setStepName("")
                storageSlot.removeItem()
                removeSlotFromQueue(storageSlot)
            end
        )
    end
end


if isAutochefEnabled() then
    for idx = 1,4 do
        local instantModSlotId = "TK_ChoppedStorageSlotInstantAutoMod"..idx

        registerMessageHandler("kStepApplied", instantModSlotId, {},
            function()
                sendStationMsgWithParamsAndUserData("kActivate", { "LuaChefChoppingStart" }, {})
            end
        )
    end

    registerMessageHandler("kTutorialStarted","",{},
        function()
            sendStationMsgWithParamsAndUserData("kActivate", { "LuaEnableRegularChoppingSlots" }, {})
        end
    )

    registerMessageHandler("kTutorialEnded","",{},
        function()
            sendStationMsgWithParamsAndUserData("kActivate", { "LuaDisableRegularChoppingSlots" }, {})
        end
    )

    registerMessageHandler("kTutorialStarted","",{},
        function()
            tutorialActive = true
            sendStationMsgWithParamsAndUserData("kActivate", { "LuaEnableRegularChoppingSlots" }, {})
        end
    )

    registerMessageHandler("kTutorialEnded","",{},
        function()
            tutorialActive = false
            sendStationMsgWithParamsAndUserData("kActivate", { "LuaDisableRegularChoppingSlots" }, {})
        end
    )

    registerMessageHandler("kStarted", "TK_ChoppedModSlotTimer", {},
        function()
            if tutorialActive then
                sendStationMsgWithParamsAndUserData("kActivate", { "LuaChefChoppingStart" }, {})
            end
        end
    )

    sendStationMsgWithParamsAndUserData("kActivate", { "LuaDisableRegularChoppingSlots" }, {})
end

--  Register a message handler for when the last available storage slot is full
--      This should mean ALL of them a full, and the oldest one needs to be
--      cleared for a NEW ingredient
local maxSlotName = "TK_ChoppedStorageSlot"..maxStorageSlots
registerMessageHandler("kFull",maxSlotName,{},
    function()
        bumpOldestInQueue()
        sendStationMsgWithParamsAndUserData("kActivate", { "LuaMadeStorageAvailableNope" }, {})
    end
)
-------------------------------------------------------------------------------------------------
-- Chopping Station End
-------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------
-- Shaker Station Start
-------------------------------------------------------------------------------------------------
local currentBaseId = ""
local currentModString = ""
local currentMods = {}
local activeShakers = 0
local shakerFinalMod = "TK_Lever"


local shakerIngredientIds   =
    {"TK_Strawberry","TK_Pineapple","TK_Acai","TK_Orange","TK_ChocolateCoffeeBeans","TK_Coconut","TK_LiHingCandy"}
local shakerMods            = {"TK_ShakewithStrawberryJuice","TK_ShakewithPineappleJuice","TK_ShakewithAcaiJuice","TK_ShakewithOrangeJuice","TK_ShakewithChocolateCoffeeBeans","TK_ShakewithCoconutMilk","TK_ShakewithLiHingCandy"}
local shakerIngredientToModMap = {}
local shakerIngredientSortPriority = {}

local currentShakerInputIds = {}
local shakerInputSlots = {}
local shakerOutputSlots = {}

for idx = 1, #shakerIngredientIds do
    local ingredientId = shakerIngredientIds[idx]
    shakerIngredientToModMap[ingredientId] = shakerMods[idx]
    shakerIngredientSortPriority[ingredientId] = idx
end

local getFreeShaker = function()
    for idx = 1, #shakerOutputSlots do
        local shaker = shakerOutputSlots[idx]
        if shaker.isAvailable() then
            return shaker
        end
    end
    return nil
end

local modStringFromMods = function(mods)
    local modString = ""
    if #mods > 0 then
        modString = modString..mods[1]
    end
    if #mods > 1 then
        for idx = 2, #mods do
            modString = modString .. "," .. mods[idx]
        end
    end
    return modString
end

local checkForValidCombos = function(inputIngredients, mustUseAll)
    sendStationMsgWithParamsAndUserData("kActivate", { "Lua","checkForValidCombos started, mustUseAll",mustUseAll}, {})
    local freeShaker = getFreeShaker()
    --Now that we have a sorted list of ingredient ids, it should be easy to try all the different combos!
    for idx = 1, #inputIngredients do
        local baseIngredient = inputIngredients[idx]
        local baseIngredientId = baseIngredient.getFirstIngredientID()
        local baseIngredientMod = shakerIngredientToModMap[baseIngredientId]
        local baseIngredientAppliedMods = modStringFromMods(baseIngredient.getFirstIngredientModIDs())
        if baseIngredientAppliedMods ~= "" then
            baseIngredientAppliedMods = baseIngredientAppliedMods..","
        end

        local modString = ""
        local mods = {}

        -- build a modstring using ALL the other ingredients
        --      but also test for valid single mod combos

        sendStationMsgWithParamsAndUserData("kActivate", { "Lua","getValidShakerCombo--checking ingredientId",baseIngredientId}, {})
        local singleMod = ""
        for idx2 = 1, #inputIngredients do
            --Skip the base ingredient
            local testIngredient = inputIngredients[idx2]
            local testIngredientId = testIngredient.getFirstIngredientID()
            local ingredientMod = shakerIngredientToModMap[testIngredientId]
            sendStationMsgWithParamsAndUserData("kActivate", { "Lua","getValidShakerCombo--checking baseIngredientId with testIngredientId",baseIngredientId,testIngredientId}, {})
            if baseIngredientMod ~= ingredientMod then
                sendStationMsgWithParamsAndUserData("kActivate", { "Lua","checkForValidCombos--inserting to table",ingredientMod}, {})
                table.insert(mods, ingredientMod)
                sendStationMsgWithParamsAndUserData("kActivate", { "Lua","checkForValidCombos--inserted to table",ingredientMod}, {})
                if mustUseAll < 1 and singleMod == "" and freeShaker.couldAcceptIngredientWithStepAndModString(baseIngredientId,shakerFinalMod,baseIngredientAppliedMods..ingredientMod) then
                    singleMod = ingredientMod
                    sendStationMsgWithParamsAndUserData("kActivate", { "Lua","checkForValidCombos--singleMod found",singleMod}, {})
                end
            end
        end

        local modString = modStringFromMods(mods)
        sendStationMsgWithParamsAndUserData("kActivate", { "Lua","checkForValidCombos--baseIngredientId,modstring",baseIngredientId,modString}, {})


        --Now test the modString of ALL the other ingredients first, then return any successful single mods
        if modString ~= "" and freeShaker.couldAcceptIngredientWithStepAndModString(baseIngredientId,shakerFinalMod,baseIngredientAppliedMods..modString) then
            currentBaseId = baseIngredientId
            currentModString = baseIngredientAppliedMods..modString
            currentMods = mods
            sendStationMsgWithParamsAndUserData("kActivate", { "Lua","validShakerCombo--baseIngredientId,modString",baseIngredientId,modString}, {})
            return baseIngredient
        end
        if singleMod ~= "" then
            currentBaseId = baseIngredientId
            currentModString = baseIngredientAppliedMods..singleMod
            table.insert(currentMods, singleMod)
            sendStationMsgWithParamsAndUserData("kActivate", { "Lua","validShakerCombo--baseIngredientId,singleMod",baseIngredientId,baseIngredientAppliedMods..singleMod}, {})
            return baseIngredient
        end
    end

    return nil
end

local getValidShakerCombo = function()

    sendStationMsgWithParamsAndUserData("kActivate", { "Lua","getValidShakerCombo started"}, {})
    currentBaseId = ""
    currentModString = ""
    currentMods = {}

    local freeShaker = getFreeShaker()
    if freeShaker == nil then
        return nil
    end

    --For now, I'm assuming that the same stored ingredient id will always apply the same mod in the shaker
    local inputIngredients = {}
    local unusedInputIngredientIds = {}
    for idx = 1,#shakerIngredientIds do
        local foodItemId = shakerIngredientIds[idx]
        unusedInputIngredientIds[foodItemId] = true
    end

    for idx = 1,#shakerInputSlots do
        local inputSlot = shakerInputSlots[idx]
        if inputSlot.hasFoodItem() then
            local foodItem = inputSlot.getFoodItem()
            if not isNullRef(foodItem) then
                local foodItemId = foodItem.getFirstIngredientID()

                --Also assume a given ingredient id will only be used once
                if unusedInputIngredientIds[foodItemId] then
                    table.insert(inputIngredients, foodItem)
                    unusedInputIngredientIds[foodItemId] = false
                end
            end
        end
    end

    local inputsCount = #inputIngredients
    sendStationMsgWithParamsAndUserData("kActivate", { "Lua","inputsCount", inputsCount}, {})

    if inputsCount < 2 then
        return nil
    end

    table.sort(inputIngredients,
        function(idA, idB)
            return shakerIngredientSortPriority[idA.getFirstIngredientID()] < shakerIngredientSortPriority[idB.getFirstIngredientID()]
    end)

    sendStationMsgWithParamsAndUserData("kActivate", { "Lua","getValidShakerCombo--ids sorted"}, {})

    local ingredientFound = checkForValidCombos(inputIngredients, 1)
    if currentBaseId == "" then
        ingredientFound = checkForValidCombos(inputIngredients, 0)
    end
    return ingredientFound
end

local clearFirstInputWithMod = function(mod)
    for idx = 1,#shakerInputSlots do
        local inputSlot = shakerInputSlots[idx]
        if inputSlot.hasFoodItem() then
            local foodId = inputSlot.getFoodItem().getFirstIngredientID()
            local modId = shakerIngredientToModMap[foodId]
            sendStationMsgWithParamsAndUserData("kActivate", { "Lua","checkingForMod--mod,slotMod",mod,modId}, {})
            if modId == mod then
                print("modFound")
                inputSlot.removeItem()
                return true
            end
        end
    end
    return false
end

local clearInputSlots = function(mods)
    sendStationMsgWithParamsAndUserData("kActivate", { "Lua","clearFirstInputWithMod--mod",shakerIngredientToModMap[currentBaseId]}, {})
    local foundMod = clearFirstInputWithMod(shakerIngredientToModMap[currentBaseId])
    for mdx = 1,#mods do
        local mod = mods[mdx]
        sendStationMsgWithParamsAndUserData("kActivate", { "Lua","clearFirstInputWithMod--mod",mod}, {})
        foundMod = clearFirstInputWithMod(mod)
    end
end

for idx = 1,3 do
    local inputSlotName = "ShakerInputSlot"..idx
    local inputStorageSlot = getStationWithId(inputSlotName)

    registerMessageHandler("kActivate",inputSlotName.."TapArea",{"visit"},
        function()
            --MOVED THE VISIT BEHAVIOR TO LUA TO GUARANTEE THE ORDER OF EXECUTION
            inputStorageSlot.doFloVisit()
            --check for valid combos and change the lever's ready state to match
            getValidShakerCombo()
            if currentBaseId ~= "" then
                sendStationMsgWithParamsAndUserData("kActivate", { "LuaEnableShakerStart" }, {})
            else
                sendStationMsgWithParamsAndUserData("kActivate", { "LuaDisableShakerStart" }, {})
            end
        end
    )

    shakerInputSlots[idx] = inputStorageSlot

    local outputSlotName = "ShakerSlot"..idx
    if idx <= kNormalShakers or autochefEnabled then
        local outputSlot = getStationWithId(outputSlotName)
        if outputSlot.isUnlocked() then
            registerMessageHandler("kIngredientOut",outputSlotName,{},
                function()
                    --check for valid combos
                    getValidShakerCombo()
                    if currentBaseId ~= "" then
                        sendStationMsgWithParamsAndUserData("kActivate", { "LuaEnableShakerStart", outputSlotName}, {})
                    end
                end
            )

            -- trigger the chef animations based on the shaker animation slots
            local animName = outputSlotName.."Anim"
            registerMessageHandler("kStarted",animName,{"intro"},
                function()
                    if activeShakers == 0 then
                        sendStationMsgWithParamsAndUserData("kActivate", { "LuaStartChefShaking", outputSlotName}, {})
                    end
                    activeShakers = activeShakers + 1
                end
            )

            registerMessageHandler("kStarted",animName,{"outro"},
                function()
                    activeShakers = activeShakers - 1
                    if activeShakers <= 0 then
                        sendStationMsgWithParamsAndUserData("kActivate", { "LuaStopChefShaking", outputSlotName}, {})
                    end
                end
            )

            shakerOutputSlots[idx] = outputSlot
        end
    end
end

registerMessageHandler("kActivate","ShakerLeverTapArea",{"visit"},
    function()
        local ingredient = getValidShakerCombo()
        local shakerSlot = getFreeShaker()
        print("currentBaseId:" .. currentBaseId .. " currentModString:" .. currentModString)
        if currentBaseId ~= "" and currentModString ~= "" and shakerSlot ~= nil then
            sendStationMsgWithParamsAndUserData("kActivate", { "LuaAnimateLever"}, {})
            shakerSlot.startWithIngredientAndStepAndModString(currentBaseId,shakerFinalMod,currentModString)
            local mods = {}
            for idx = 1,#currentMods do
                mods[idx] = currentMods[idx]
            end
            clearInputSlots(mods)
        else
            sendStationMsgWithParamsAndUserData("kActivate", { "LuaFailedShakerStart" }, {})
        end
    end
)

-------------------------------------------------------------------------------------------------
-- Shaker Station End
-------------------------------------------------------------------------------------------------

-------------------------------------------------------------------------------------------------
-- Per frame updates
-------------------------------------------------------------------------------------------------
setUpdateFunction(
	function (timeDelta)
		-- If the venue has not started, do nothing
		if not venueStarted then
			return
		end
	end
)

