-------------------------------------------------------------------------------------------------
-- Recording when the venue has started
-------------------------------------------------------------------------------------------------
local venueStarted = false
local   mRandomTriggerDelay             = 2
local   mRandomTriggerCounter           = mRandomTriggerDelay
local autochefEnabled = isAutochefEnabled()
local currentTutorial = getCurrentTutorial()
local noTutorialActive = currentTutorial == ""
local getNoTutorialActive = function()
    local currentTutorial = getCurrentTutorial()
    local noTutorialActive = currentTutorial == ""
    return noTutorialActive
end

local flo = getVenueInstance().getFlo()
local venueSpecific = getVenueInstance().getVenueSpecific()

-------------------------------------------------------------------------------------------------
-- Start Scooper Code
-------------------------------------------------------------------------------------------------
-- Scooper toggle references
local   scooperMod                          = "BD_BMod"
local   toggleIdleSprite                    = "venue_27/flashatlas_v27/stations.cdx_effect_bagel_highlight.idle"
local   maxTogglesActive                    = 4
local   currentTogglesActive                = 0
local   activeMods                          = {}
local   currentToggleMods                   = ""
local   toggleSlotIngredients               = {"BD_CreamCheese","BD_Lox","BD_Capers","BD_Onions"}
local   toggleSlotLabels                    = {"creamcheese","lox","capers","onions"}
local   toggleSlotCookieSFX                 = {"bigbelly_cookie_creamcheese_loop","bigbelly_cookie_lox_toss_loop","bigbelly_cookie_isaac_capers","bigbelly_cookie_isaac_onions"}
local   toggleSlotIsaacSFX                  = {"bigbelly_isaac_creamcheese","bigbelly_isaac_lox","bigbelly_cookie_isaac_capers","bigbelly_cookie_isaac_onions"}
local   toggleSlotIngredientsToLabels       = {}
local   toggleSlotIngredientsToSFX          = {}
local   chefAnimActiveLabels                = {}
local   toggleSlots                         = {}
local   toggleSlotTapAreas                  = {}
local   toggleSlotTapAreaToIndicator        = {}
local   toggleSlotTapAreaToOldIndicator     = {}
local   toggleSlotTapAreaBoolMap            = {}
local   toggleSlotTapAreaToIngredientIDs    = {}
local   scooperSlotNames                    = {"BD_BModSlot0","BD_BModSlot1","BD_BModSlot2"};
local   scooperSlots                        = {}
local   scooperSlotTapAreas                 = {}
local   scooperSlotTapAreaToSlots           = {}

local chefAnimId = "ChefTorsoAnim"
local chefLegsAnimId = "ChefLegsAnim"
if autochefEnabled then
    chefAnimId = "AutoChefTorsoAnim"
    chefLegsAnimId = "AutoChefLegsAnim"
end
local   chefTorsoAnim                       = getStationWithId(chefAnimId)
local   chefLegsAnim                        = getStationWithId(chefLegsAnimId)

-- Grab the toggle nodes
for idx = 1, #toggleSlotIngredients do
    local toggleSlotIngredient = toggleSlotIngredients[idx]
    local toggleSlot = getStationWithId(toggleSlotIngredient)
    local toggleSlotTapArea = getStationWithId(toggleSlotIngredient.."TapArea")
    local toggleSlotIndicator = getStationWithId(toggleSlotIngredient.."Indicator")
    local toggleSlotOldIndicator = getStationWithId(toggleSlotIngredient.."OldIndicator")
    local toggleSlotLabel = toggleSlotLabels[idx]

    toggleSlots[#toggleSlots + 1]                           = toggleSlot
    toggleSlotTapAreas[#toggleSlotTapAreas + 1]             = toggleSlotTapArea
    toggleSlotTapAreaToIndicator[toggleSlotTapArea]         = toggleSlotIndicator
    toggleSlotTapAreaToOldIndicator[toggleSlotTapArea]      = toggleSlotOldIndicator
    toggleSlotTapAreaToIngredientIDs[toggleSlotTapArea]     = toggleSlotIngredients[idx]
    toggleSlotIngredientsToLabels[toggleSlotIngredient]     = toggleSlotLabel
    chefAnimActiveLabels[toggleSlotLabel]                   = 0
    toggleSlotTapAreaBoolMap[toggleSlotTapArea]             = false
end

-- Grab the scooper slot nodes
for idx = 1, #scooperSlotNames do
    local scooperSlot = getStationWithId(scooperSlotNames[idx])
    local scooperSlotTapArea = getStationWithId(scooperSlotNames[idx].."TapArea")

    scooperSlots[#scooperSlots + 1]                           = scooperSlot
    scooperSlotTapAreas[#scooperSlotTapAreas + 1]             = scooperSlotTapArea
    scooperSlotTapAreaToSlots[scooperSlotTapArea]             = scooperSlot
end

-- Change a vector of mods into a string with the format
-- "mod1,mod2,mod3"
local getModStringForMods = function(modVector)
    local modString = ""
    for modIdx = 1, #modVector do
        if modIdx == 1 then
            modString = modVector[modIdx]
        else
            modString = modString..","
            modString = modString..modVector[modIdx]
        end
    end
    return modString
end

local getAutochefLabelForMods = function(modVector)
    local modString = "idle_1"
    local totalMods = 0
    for modIdx = 1, #modVector do
            local toggleLabel = toggleSlotIngredientsToLabels[modVector[modIdx]]
            modString = toggleLabel
    end
    return modString
end

-- Updates the current mod string based on the toggled slots
local updateModsForToggleState = function()
    activeMods = {}
    for idx = 1, #toggleSlotTapAreas do
        local tapArea = toggleSlotTapAreas[idx]
        local toggleModId = toggleSlotTapAreaToIngredientIDs[tapArea]
        if toggleSlotTapAreaBoolMap[tapArea] then
            activeMods[#activeMods+1] = toggleModId
        end
    end
    currentToggleMods = getModStringForMods(activeMods)
end

local doToggleSlot = function(tapArea)
    local labelString = "idle"
    if toggleSlotTapAreaBoolMap[tapArea] then
        labelString = "idle"
        currentTogglesActive = currentTogglesActive - 1
        toggleSlotTapAreaBoolMap[tapArea] = not (toggleSlotTapAreaBoolMap[tapArea] or false)
    elseif maxTogglesActive > currentTogglesActive and not toggleSlotTapAreaBoolMap[tapArea] then
        labelString = "active"
        currentTogglesActive = currentTogglesActive + 1
        toggleSlotTapAreaBoolMap[tapArea] = not (toggleSlotTapAreaBoolMap[tapArea] or false)
    elseif maxTogglesActive <= currentTogglesActive then
        sendStationMsgWithParamsAndUserData("kActivate", { "MaxFlavors" }, {})
    end
    return labelString
end

local triggerChefAnimForToggles = function()
    if not isNullRef(chefTorsoAnim) then
        --for each active mod, add a label to the regular chef
        for idx = 1, #activeMods do
            local ingredient = activeMods[idx]
            local label = toggleSlotIngredientsToLabels[ingredient]
            local labelCount = chefAnimActiveLabels[label]
            chefAnimActiveLabels[label] = labelCount + 1
            chefTorsoAnim.addAnimation(label)
        end
    end
end

local clearToggleStates = function()
    for idx = 1, #toggleSlotTapAreas do
        local tapArea = toggleSlotTapAreas[idx]
        local toggleModId = toggleSlotTapAreaToIngredientIDs[tapArea]
        local indicator = toggleSlotTapAreaToIndicator[tapArea]
        local indicatorOld = toggleSlotTapAreaToOldIndicator[tapArea]

        toggleSlotTapAreaBoolMap[tapArea] = false
        if not isNullRef(indicator) then
            indicator.startAnimation({"idle"},true)
        end
        sendStationMsgWithParamsAndUserData("kVenueSpecific", { toggleModId, "idle" }, { currentToggleMods })

    end
    currentTogglesActive = 0
    currentToggleMods = ""
    activeMods = {}
    sendStationMsgWithParamsAndUserData("kVenueSpecific", { toggleModId, "idle" }, { currentToggleMods })
end

--Return first item carried by flo that can have the currently toggled mods applied to it by this applyNSteps slot
--  if one is not found, return nil
local function getAcceptableCarriedItem(slot)
    local handCount = flo.getOccupiedHandCount()
    if handCount > 0 then
        for mapIdx = 0,handCount do
            local foodItem = flo.getFoodFromHandByIndex(mapIdx)
            if not isNullRef(foodItem) then
                local baseId = foodItem.getFirstIngredientID()
                local mods = foodItem.getFirstIngredientModIDs()

                local modString = getModStringForMods(mods)
                modString = modString..","
                modString = modString..currentToggleMods

                local foundValid = slot.couldAcceptIngredientWithStepAndModString(baseId, scooperMod, modString)
                if foundValid then
                    return foodItem
                end
            end
        end
    end

    return nil
end

local countTable = function(table)
    local count = 0
    for idx=1,#table do
        count = count +1
    end
    return count
end

local function getAutochefModsForSlot(slot)
    local modsForSlot = {}
    local handCount = flo.getOccupiedHandCount()
    if handCount > 0 then
        for mapIdx = 0,handCount do
            local foodItem = flo.getFoodFromHandByIndex(mapIdx)
            if not isNullRef(foodItem) then
                local autoChefMods = venueSpecific.getNextApplicableMods(foodItem)
                if countTable(autoChefMods) > 0 then
                    return autoChefMods
                end
            end
        end
    end
    return modsForSlot
end

local triggerChefAnimForMods = function(mods)
    if not isNullRef(chefTorsoAnim) then
        if autochefEnabled then
            local label = getAutochefLabelForMods(mods)
            if label == "idle_1" then
                return
            end
            chefTorsoAnim.startAnimation({label},false)
            chefLegsAnim.startAnimation({label},false)
            local numMods = countTable(mods)

            local sfxLabel = ""
            --trigger Isaac's sfx
            if numMods >= 1 then
                sfxLabel = mods[1]
            end
            sendStationMsgWithParamsAndUserData("kActivate", { sfxLabel }, {})

        else
            --for each active mod, add a label to the regular chef
            for idx = 1, #mods do
                local ingredient = mods[idx]
                local label = toggleSlotIngredientsToLabels[ingredient]
                local labelCount = chefAnimActiveLabels[label]
                chefAnimActiveLabels[label] = labelCount + 1
                chefTorsoAnim.addAnimation(label)
            end
        end
    end
end

--What happens when you tap a toggle...
for idx = 1, #toggleSlotTapAreas do
    local tapArea = toggleSlotTapAreas[idx]
    local slotIngredientID = toggleSlotTapAreaToIngredientIDs[tapArea]
    registerMessageHandler("kActivate", tapArea.getStationNodeId(), {"tap"},
        function()
            if autochefEnabled and noTutorialActive then
                sendStationMsgWithParamsAndUserData("kActivate", { "ToggleTapped" }, {})
            else
                --Firstpass logic, can we control these correctly, and toggle them via script
                local labelString = doToggleSlot(tapArea)
                updateModsForToggleState()
                if noTutorialActive and autochefEnabled then
                else
                    sendStationMsgWithParamsAndUserData("kVenueSpecific", { slotIngredientID, labelString }, { currentToggleMods })
                end
            end
        end
    )
end

--What happens when you tap a scooper slot...
for idx = 1, #scooperSlotTapAreas do
    local tapArea = scooperSlotTapAreas[idx]
    registerMessageHandler("kActivate", tapArea.getStationNodeId(), {"visit"},
        function()
            local slot              = scooperSlotTapAreaToSlots[tapArea]
            local slotOccupied      = slot.peekItemAvailableForRemoval()

            if autochefEnabled and noTutorialActive and not isNullRef(chefTorsoAnim) then
                --local autoChefMods = activeMods
                local autoChefMods = getAutochefModsForSlot(slot)
                local autoChefModString = getModStringForMods(autoChefMods)
                autoChefModString = autoChefModString..","
                autoChefModString = autoChefModString..scooperMod
                slot.setStepsFromModString(autoChefModString)
                triggerChefAnimForMods(autoChefMods)
                slot.doFloVisit()
                return
            end

            --Early out if nothing is toggled
            if currentToggleMods == "" and isNullRef(slotOccupied) then
                sendStationMsgWithParamsAndUserData("kActivate", { "SelectToggles" }, {})
                return
            end

            local shouldClearState = false
            local acceptableItem = getAcceptableCarriedItem(slot)
            if acceptableItem then
                local allMods = currentToggleMods..","
                allMods = allMods..scooperMod
                slot.setStepsFromModString(allMods)
                slot.doFloVisit()
            elseif not isNullRef(slotOccupied) then
                --Pickup of finished item
                slot.doFloVisit()
            else
                --sous chef tip trigger
                sendStationMsgWithParamsAndUserData("kActivate", { "NotServedHere" }, {})
            end
        end
    )
end

--Scooper slot effects
--On ingredientIn, clear the toggle states of all the ice cream flavors
for idx = 1,#scooperSlots do
    local scooperSlot = scooperSlots[idx]

    --register a handler for each flavor
    --On step applied, clean up regular chef anim state for that flavor
    for idx = 1, #toggleSlotIngredients do
        local ingredient = toggleSlotIngredients[idx]
        local label = toggleSlotIngredientsToLabels[ingredient]
        registerMessageHandler("kStepApplied",scooperSlot.getStationNodeId(),{toggleSlotIngredients[idx]},
            function()

                --turn off the regular chef animations as necessary
                if not autochefEnabled then
                    chefAnimActiveLabels[label] = chefAnimActiveLabels[label] - 1
                    if chefAnimActiveLabels[label] <= 0 then
                        chefAnimActiveLabels[label] = 0
                        chefTorsoAnim.removeAnimation(label)
                    end
                end

                --clear the multiple steps from the slot tapped (technically only needs to happen once)
                scooperSlot.setStepsFromModString("")
            end
        )
    end


    --When an ingredient is added to a scooper, clear the toggle states
    registerMessageHandler("kIngredientIn",scooperSlot.getStationNodeId(),{},
        function()
            if not autochefEnabled then
                triggerChefAnimForMods(activeMods)
            end
            clearToggleStates()
        end
    )

end

-------------------------------------------------------------------------------------------------
-- End Scooper Code
-------------------------------------------------------------------------------------------------

registerMessageHandler("kTutorialStarted","",{},
    function()
        noTutorialActive = false
    end
)

registerMessageHandler("kTutorialEnded","",{},
    function()
        noTutorialActive = true
        if venueStarted then
            local i = 0
        end
    end
)

-------------------------------------------------------------------------------------------------
-- Per frame updates
-------------------------------------------------------------------------------------------------
setUpdateFunction(
	function (timeDelta)
		-- If the venue has not started, do nothing
		if not venueStarted then
			return
		end

        mRandomTriggerCounter = mRandomTriggerCounter - timeDelta

        if mRandomTriggerCounter <= 0 then
            -- Enough time has passed to trigger Gnarly
            mRandomTriggerCounter = mRandomTriggerDelay
        end


	end
)

