-------------------------------------------------------------------------------------------------
-------------------------------------------------------------------------------------------------
-- Recording when the venue has started
-------------------------------------------------------------------------------------------------
local venueStarted = false
local autochefEnabled = isAutochefEnabled()

local currentTutorial = getCurrentTutorial()
local noTutorialActive = currentTutorial == ""

local getNoTutorialActive = function()
    local testTutorial = getCurrentTutorial()
    local noTutorialActive = testTutorial == ""
    return noTutorialActive
end

local venueItem = getVenueInstance()
local flo = venueItem.getFlo()

-------------------------------------------------------------------------------------------------
-- Fire Starting stuff
-------------------------------------------------------------------------------------------------
local floAnim = flo.getFloAnimation()
local fireIntroTime = floAnim.getAnimationTimeForLabel("lighting_intro")
local fireOutroTime = floAnim.getAnimationTimeForLabel("lighting_outro")

local starterMods = {"AK_SoupPot","AK_Melt","AK_Roast"}
local starterAutoAnims = {"fire_soup","fire_melting","fire_roasting"}
local starterUnlockedAutoAnims = {}
local unlockedCount = 0
local startedArray = {false, false, false}

for idx = 1, #starterMods do
    local starterMod = starterMods[idx]

    registerMessageHandler("kActivate",starterMod.."StarterTapArea",{"visit"},
        function()
            if not(startedArray[idx]) then
                startedArray[idx] = true
                flo.startTimeBasedTask(fireIntroTime,"kFloState_StartingFireIntro")
                sendStationMsgWithParamsAndUserData("kActivate", {"LuaFireStart",starterMod}, {})
            end
        end
    )
    registerMessageHandler("kEnded",starterMod.."StarterTimer",{},
        function()
            startedArray[idx] = true
            flo.startTimeBasedTask(fireOutroTime,"kFloState_StartingFireOutro")
        end
    )

    local modStation = getStationWithId(starterMod.."1ModSlot")
    if modStation.isUnlocked() then
        table.insert(starterUnlockedAutoAnims,starterAutoAnims[idx])
        unlockedCount = unlockedCount + 1
    end
end

-------------------------------------------------------------------------------------------------
-- General Setup
-------------------------------------------------------------------------------------------------
registerMessageHandler("kVenueStarted","",{},
    function()
        venueStarted = true
        local noTutorialActive =  getNoTutorialActive()
        if noTutorialActive then
            sendStationMsgWithParamsAndUserData("kActivate", {"LuaInitialFill","TutorialEnded"}, {})
            if autochefEnabled then
                sendStationMsgWithParamsAndUserData("kActivate", {"LuaRefillCococups","NoTutorial"}, {})
            end
        end
    end
)

registerMessageHandler("kTutorialEnded","",{},
    function()
        local noTutorialActive =  getNoTutorialActive()
        if noTutorialActive and venueStarted then
            sendStationMsgWithParamsAndUserData("kActivate", {"LuaInitialFill","TutorialEnded"}, {})
            if autochefEnabled then
                sendStationMsgWithParamsAndUserData("kActivate", {"LuaRefillCococups","TutorialEnded"}, {})
            end
        end
    end
)
